# -*- coding: utf-8 -*-
import math

from datetime import datetime, timedelta
from odoo import api, models, SUPERUSER_ID
from odoo.addons.saas_worker import metabase
from odoo.addons.saas_worker.util import version_gte
from odoo.exceptions import AccessError
from odoo.tools import DEFAULT_SERVER_DATETIME_FORMAT
from odoo.tools.translate import _


class IrMinCron(models.AbstractModel):
    _name = 'ir.min.cron.mixin'
    _description = "Set Min Next call on metabase"

    @api.model
    def _set_meta_cron_min_time(self):
        cr = self._cr
        db = metabase.get(cr.dbname)
        if db:
            if version_gte("saas~14.2"):
                cr.execute("""
                    select min(least(t.call_at, c.nextcall)) from ir_cron c left join ir_cron_trigger t on (t.cron_id = c.id) where c.active;
                """)
            else:
                cr.execute("select min(nextcall) from ir_cron where active = 't'")
            min_cron = cr.fetchone()[0]
            if min_cron:
                if isinstance(min_cron, str):
                    min_cron = datetime.strptime(min_cron, DEFAULT_SERVER_DATETIME_FORMAT)
                if min_cron < datetime.now():
                    min_cron = datetime.now()
            else:
                min_cron = datetime.now() + timedelta(hours=1)
            if db.mode in ['trial']:
                # For non-paid databases, add hours of delay per week of non-connections
                idle_weeks = int((datetime.now() - (db.last_cnx_date or db.create_date)).days / 7)
                min_cron = min_cron + timedelta(hours=80 + 80 * math.tanh(idle_weeks - 4))
            metabase.update(db, cron_time=min_cron.strftime(DEFAULT_SERVER_DATETIME_FORMAT))

    @api.model
    def create(self, values):
        res = super(IrMinCron, self).create(values)
        self._set_meta_cron_min_time()
        return res

    @api.multi
    def write(self, values):
        res = super(IrMinCron, self).write(values)
        self._set_meta_cron_min_time()
        return res

    @api.multi
    def unlink(self):
        res = super(IrMinCron, self).unlink()
        self._set_meta_cron_min_time()
        return res


class IrCron(models.Model):
    _inherit = ['ir.cron', 'ir.min.cron.mixin']
    _name = 'ir.cron'

    @api.multi
    def write(self, values):
        ping_cron = self.env.ref('mail.ir_cron_module_update_notification', raise_if_not_found=False)
        if ping_cron and ping_cron in self and list(values) != ['lastcall']:
            raise AccessError(_('The Update Notification scheduled action cannot be modified'))
        vacuum_cron = self.env.ref("base.autovacuum_job", raise_if_not_found=False)
        if vacuum_cron and vacuum_cron in self:
            raise AccessError(_("The Auto-vacuum scheduled action cannot be modified"))
        return super(IrCron, self).write(values)

    @api.multi
    def unlink(self):
        ping_cron = self.env.ref('mail.ir_cron_module_update_notification', raise_if_not_found=False)
        if ping_cron and ping_cron in self:
            raise AccessError(_('The Update Notification scheduled action cannot be deleted'))
        vacuum_cron = self.env.ref("base.autovacuum_job", raise_if_not_found=False)
        if vacuum_cron and vacuum_cron in self:
            raise AccessError(_("The Auto-vacuum scheduled action cannot be deleted"))
        return super(IrCron, self).unlink()


    if version_gte("15.0"):
        @classmethod
        def _process_job(cls, db, cron_cr, job):
            super(IrCron, cls)._process_job(db, cron_cr, job)
            cron_env = api.Environment(cron_cr, SUPERUSER_ID, {})[cls._name]
            cron_env._set_meta_cron_min_time()
    else:
        @classmethod
        def _process_job(cls, job_cr, job, cron_cr):
            super(IrCron, cls)._process_job(job_cr, job, cron_cr)
            with api.Environment.manage():
                cron_env = api.Environment(cron_cr, SUPERUSER_ID, {})[cls._name]
                cron_env._set_meta_cron_min_time()


if version_gte("saas~14.2"):
    class IrCronTrigger(models.Model):
        _inherit = ['ir.cron.trigger', 'ir.min.cron.mixin']
        _name = 'ir.cron.trigger'
